#ifndef HAL_PEER_H_
#define HAL_PEER_H_

/* Macros to cast pointer to our base peer. */
#define	HAL_PEER_BASE(klass)	((struct hal_peer_base *)(klass))

/* Peer is a basic object of the hal protocol, each connection is represented
 * by this data structure. The base peer is a first field in other
 * structures that represent more sophisticated objects. */

typedef enum {
	HAL_PEER_EVENT_READ,
	HAL_PEER_EVENT_WRITE,
	HAL_PEER_EVENT_ERROR,
	HAL_PEER_EVENT_TIMEOUT
} hal_peer_event_t;

struct hal_peer_base;

typedef void (*hal_peer_event_callback_t)(struct hal_peer_base *peer,
					  hal_peer_event_t event_handler);

struct hal_peer_base {
	struct io_context	  *io;		 /* peer loop context */
	struct buffer		  *wb;		 /* buffer for sent data */
	struct buffer		  *rb;		 /* buffer for received data */
	int			  wfd;		 /* fd to write data from wb */
	int			  rfd;		 /* fd to read data for rb */
	hal_peer_event_callback_t event_handler; /* peer event handler */
};

/* Allocate resources and set data structures. */
struct hal_peer_base *hal_peer_base_new();
void	hal_peer_base_init(struct hal_peer_base *peer, struct io_context *io,
			   int wfd, int rfd, hal_peer_event_callback_t callback);
void	hal_peer_base_deinit(struct hal_peer_base *peer);
void	hal_peer_base_free(struct hal_peer_base **peer);

/* Default functions for io loop, internally it will call
 * hal_peer_event_callback. */
void    hal_peer_event_default(int fd, short events, void *opaque);

/* This function not realy write something to write descriptor it only adds
 * EVEN_WRITE to wfd. The real writting operation will be perfomed when a user
 * runs a loop. */
void	hal_peer_send_default(struct hal_peer_base *peer);

#endif
